"""testcases for cssutils.css.CSSUnkownRule"""

import xml.dom
import test_cssrule
import cssutils

class CSSUnknownRuleTestCase(test_cssrule.CSSRuleTestCase):

    def setUp(self):
        super(CSSUnknownRuleTestCase, self).setUp()
        self.r = cssutils.css.CSSUnknownRule()
        self.rRO = cssutils.css.CSSUnknownRule(readonly=True)
        self.r_type = cssutils.css.CSSUnknownRule.UNKNOWN_RULE
        self.r_typeString = 'UNKNOWN_RULE'

    def tearDown(self):
        cssutils.ser.prefs.useDefaults()
        
    def test_init(self):
        "CSSUnknownRule.type and init"
        super(CSSUnknownRuleTestCase, self).test_init()

        self.assertFalse(self.r.wellformed)

        # only name
        r = cssutils.css.CSSUnknownRule(cssText=u'@init;')
        self.assertEqual(u'@init', r.atkeyword)
        self.assertEqual(u'@init;', r.cssText)
        self.assertTrue(r.wellformed)

        # @-... not allowed?
        r = cssutils.css.CSSUnknownRule(cssText=u'@-init;')
        self.assertEqual(u'@-init;', r.cssText)
        self.assertEqual(u'@-init', r.atkeyword)
        self.assertTrue(r.wellformed)

        r = cssutils.css.CSSUnknownRule(cssText=u'@_w-h-a-012;')
        self.assertEqual(u'@_w-h-a-012;', r.cssText)
        self.assertEqual(u'@_w-h-a-012', r.atkeyword)
        self.assertTrue(r.wellformed)

        # name and content
        r = cssutils.css.CSSUnknownRule(cssText=u'@init xxx;')
        self.assertEqual(u'@init', r.atkeyword)
        self.assertEqual(u'@init xxx;', r.cssText)
        self.assertTrue(r.wellformed)

        # name and block
        r = cssutils.css.CSSUnknownRule(cssText=u'@init { xxx }')
        self.assertEqual(u'@init', r.atkeyword)
        self.assertEqual(u'@init {\n    xxx\n    }', r.cssText)
        self.assertTrue(r.wellformed)

        # name and content and block
        r = cssutils.css.CSSUnknownRule(cssText=u'@init xxx { yyy }')
        self.assertEqual(u'@init', r.atkeyword)
        self.assertEqual(u'@init xxx {\n    yyy\n    }', r.cssText)
        self.assertTrue(r.wellformed)

    def test_cssText(self):
        "CSSUnknownRule.cssText"
        tests = {
            # not normal rules!
            u'@font-facex{}': u'@font-facex {\n    }',
            u'@importurl(x.css);': u'@importurl (x . css);',
            u'@mediaAll{}': u'@mediaall {\n    }',
            u'@namespacep"x";': u'@namespacep "x";',
            u'@pageX{}': u'@pagex {\n    }',
            u'@xbottom { content: counter(page) }': u'@xbottom {\n    content: counter(page)\n    }', 
            u'@xbottom { content: "x" counter(page) "y"}': u'@xbottom {\n    content: "x" counter(page) "y"\n    }' 
            }
        self.do_equal_p(tests)
        
        # expects the same atkeyword for self.r so do a new one each test
        oldr = self.r
        for t, e in tests.items():
            self.r = cssutils.css.CSSUnknownRule()
            self.do_equal_r({t:e})
        self.r = oldr

        tests = {
            '@x;': None,
            '@x {}': u'@x {\n    }',
            '@x{ \n \t \f\r}': u'@x {\n    }',
            '@x {\n    [()]([ {\n        }]) {\n        }\n    }': None,
            '@x {\n    @b;\n    }': None,
            '''@x {
    @b {
        x: 1x;
        y: 2y;
        }
    }''': None,
            '@x "string" url(x);': None,
            
            # comments
            '@x/*1*//*2*/"str"/*3*//*4*/url("x");': 
                '@x /*1*/ /*2*/ "str" /*3*/ /*4*/ url(x);',
            # WS
            '@x"string"url("x");': '@x "string" url(x);',
            '@x\n\r\t\f "string"\n\r\t\f url(\n\r\t\f "x"\n\r\t\f )\n\r\t\f ;': 
                '@x "string" url(x);',
        }
        self.do_equal_p(tests)
        self.do_equal_r(tests)

        tests = {
                 u'@;': xml.dom.InvalidModificationErr,
                 u'@{}': xml.dom.InvalidModificationErr,
                 u'@ ;': xml.dom.InvalidModificationErr,
                 u'@ {};': xml.dom.InvalidModificationErr,
                 
                 u'@x ;{}': xml.dom.SyntaxErr,
                 u'@x ;;': xml.dom.SyntaxErr,
                 u'@x }  ': xml.dom.SyntaxErr,
                 u'@x }  ;': xml.dom.SyntaxErr,
                 u'@x {  ': xml.dom.SyntaxErr,
                 u'@x {  ;': xml.dom.SyntaxErr,
                 u'@x ': xml.dom.SyntaxErr,
                 u'@x (;': xml.dom.SyntaxErr,
                 u'@x );': xml.dom.SyntaxErr,
                 u'@x [;': xml.dom.SyntaxErr,
                 u'@x ];': xml.dom.SyntaxErr,
                 u'@x {[(]()}': xml.dom.SyntaxErr,
                 # trailing
                 u'@x{}{}': xml.dom.SyntaxErr,
                 u'@x{};': xml.dom.SyntaxErr,
                 u'@x{}1': xml.dom.SyntaxErr,
                 u'@x{} ': xml.dom.SyntaxErr,
                 u'@x{}/**/': xml.dom.SyntaxErr,
                 u'@x;1': xml.dom.SyntaxErr,
                 u'@x; ': xml.dom.SyntaxErr,
                 u'@x;/**/': xml.dom.SyntaxErr,

                 }
        self.do_raise_r(tests)

    def test_InvalidModificationErr(self):
        "CSSUnknownRule.cssText InvalidModificationErr"
        self._test_InvalidModificationErr(u'@unknown')

    def test_reprANDstr(self):
        "CSSUnknownRule.__repr__(), .__str__()"        
        s = cssutils.css.CSSUnknownRule(cssText='@x;')
        
        s2 = eval(repr(s))
        self.assertTrue(isinstance(s2, s.__class__))


if __name__ == '__main__':
    import unittest
    unittest.main()



"""Testcases for cssutils.css.CSSPageRule"""

import xml.dom
import test_cssrule
import cssutils

class CSSPageRuleTestCase(test_cssrule.CSSRuleTestCase):

    def setUp(self):
        super(CSSPageRuleTestCase, self).setUp()
        
        cssutils.ser.prefs.useDefaults()
        self.r = cssutils.css.CSSPageRule()
        self.rRO = cssutils.css.CSSPageRule(readonly=True)
        self.r_type = cssutils.css.CSSPageRule.PAGE_RULE#
        self.r_typeString = 'PAGE_RULE'

    def tearDown(self):
        cssutils.ser.prefs.useDefaults()
            
    def test_init(self):
        "CSSPageRule.__init__()"
        super(CSSPageRuleTestCase, self).test_init()

        r = cssutils.css.CSSPageRule()
        self.assertEqual(u'', r.selectorText)
        self.assertEqual(cssutils.css.CSSStyleDeclaration, type(r.style))
        self.assertEqual(r, r.style.parentRule)

        # until any properties
        self.assertEqual(u'', r.cssText)

        # only possible to set @... similar name
        self.assertRaises(xml.dom.InvalidModificationErr, self.r._setAtkeyword, 'x')

        def checkrefs(ff):
            self.assertEqual(ff, ff.style.parentRule)
            for p in ff.style:
                self.assertEqual(ff.style, p.parent)
                
        checkrefs(cssutils.css.CSSPageRule(
                    style=cssutils.css.CSSStyleDeclaration('font-family: x')))
        
        r = cssutils.css.CSSPageRule()
        r.cssText = '@page { font-family: x }'
        checkrefs(r)
        
        r = cssutils.css.CSSPageRule()
        r.style.setProperty('font-family', 'y')
        checkrefs(r)

        r = cssutils.css.CSSPageRule()
        r.style['font-family'] = 'z'
        checkrefs(r)

        r = cssutils.css.CSSPageRule()
        r.style.fontFamily = 'a'
        checkrefs(r)

    def test_InvalidModificationErr(self):
        "CSSPageRule.cssText InvalidModificationErr"
        self._test_InvalidModificationErr(u'@page')
        tests = {
            u'@pag {}': xml.dom.InvalidModificationErr,
            }
        self.do_raise_r(tests)

    def test_incomplete(self):
        "CSSPageRule (incomplete)"
        tests = {
            u'@page :left { ':
                u'', # no } and no content
            u'@page :left { color: red':
                u'@page :left {\n    color: red\n    }', # no }
        }
        self.do_equal_p(tests) # parse

    def test_cssText(self):
        "CSSPageRule.cssText"
        EXP = u'@page %s {\n    margin: 0\n    }'
        tests = {
            u'@page {}': u'',
            u'@page:left{}': u'',
            u'@page :right {}': u'',
            u'@page {margin:0;}': u'@page {\n    margin: 0\n    }',

            u'@page name { margin: 0 }': EXP % u'name',
            u'@page name:left { margin: 0 }': EXP % u'name:left',
            u'@page name:right { margin: 0 }': EXP % u'name:right',
            u'@page name:first { margin: 0 }': EXP % u'name:first',
            u'@page :left { margin: 0 }': EXP % u':left',
            u'@page:left { margin: 0 }': EXP % u':left',
            u'@page :right { margin: 0 }': EXP % u':right',
            u'@page :first { margin: 0 }': EXP % u':first',
            u'@page :UNKNOWNIDENT { margin: 0 }': EXP % u':UNKNOWNIDENT',

            u'@PAGE:left{margin:0;}': u'@page :left {\n    margin: 0\n    }',
            u'@\\page:left{margin:0;}': u'@page :left {\n    margin: 0\n    }',

            # comments
            u'@page/*1*//*2*/:left/*3*//*4*/{margin:0;}':
                u'@page /*1*/ /*2*/ :left /*3*/ /*4*/ {\n    margin: 0\n    }',
            # WS
            u'@page:left{margin:0;}':
                u'@page :left {\n    margin: 0\n    }',
            u'@page\n\r\f\t :left\n\r\f\t {margin:0;}':
                u'@page :left {\n    margin: 0\n    }',
                
            # MarginRule
            u'@page {    @top-right {        content: "2"        }    }':
                u'@page {\n    @top-right {\n        content: "2"\n        }\n    }',
            u'@page {padding: 1cm; margin: 1cm; @top-left {content: "1"}@top-right {content: "2";left: 1}}':
                u'@page {\n    padding: 1cm;\n    margin: 1cm;\n    @top-left {\n        content: "1"\n        }\n    @top-right {\n        content: "2";\n        left: 1\n        }\n    }',
            u'@page {@top-right { content: "1a"; content: "1b"; x: 1 }@top-right { content: "2"; y: 2 }}':
                u'''@page {\n    @top-right {
        content: "1a";
        content: "1b";
        x: 1;
        content: "2";
        y: 2
        }\n    }''',
        
        }
        self.do_equal_r(tests)
        self.do_equal_p(tests)

        tests = {
            # auto is not allowed
            u'@page AUto {}': xml.dom.SyntaxErr,
            u'@page AUto:left {}': xml.dom.SyntaxErr,
            
            u'@page : {}': xml.dom.SyntaxErr,
            u'@page :/*1*/left {}': xml.dom.SyntaxErr,
            u'@page : left {}': xml.dom.SyntaxErr,
            u'@page :left :right {}': xml.dom.SyntaxErr,
            u'@page :left a {}': xml.dom.SyntaxErr,
            # no S between IDENT and PSEUDO
            u'@page a :left  {}': xml.dom.SyntaxErr,

            u'@page :left;': xml.dom.SyntaxErr,
            u'@page :left }': xml.dom.SyntaxErr,
            }
        self.do_raise_p(tests) # parse
        tests.update({
            # false selector
            u'@page :right :left {}': xml.dom.SyntaxErr, # no }
            u'@page :right X {}': xml.dom.SyntaxErr, # no }
            u'@page X Y {}': xml.dom.SyntaxErr, # no }
            
            u'@page :left {': xml.dom.SyntaxErr, # no }
            # trailing
            u'@page :left {}1': xml.dom.SyntaxErr, # no }
            u'@page :left {}/**/': xml.dom.SyntaxErr, # no }
            u'@page :left {} ': xml.dom.SyntaxErr, # no }
            })
        self.do_raise_r(tests) # set cssText

    def test_cssText2(self):
        "CSSPageRule.cssText 2"
        r = cssutils.css.CSSPageRule()
        s = u'a:left'
        r.selectorText = s 
        self.assertEqual(r.selectorText, s)

        st = 'size: a4'
        r.style = st
        self.assertEqual(r.style.cssText, st)

        # invalid selector
        self.assertRaises(xml.dom.SyntaxErr, r._setStyle, '$')
        self.assertEqual(r.selectorText, s)
        self.assertEqual(r.style.cssText, st)

        self.assertRaises(xml.dom.SyntaxErr, r._setCssText, '@page $ { color: red }')
        self.assertEqual(r.selectorText, s)
        self.assertEqual(r.style.cssText, st)


        # invalid style
        self.assertRaises(xml.dom.SyntaxErr, r._setSelectorText, '$')
        self.assertEqual(r.selectorText, s)
        self.assertEqual(r.style.cssText, st)

        self.assertRaises(xml.dom.SyntaxErr, r._setCssText, '@page b:right { x }')
        self.assertEqual(r.selectorText, s)
        self.assertEqual(r.style.cssText, st)

    def test_selectorText(self):
        "CSSPageRule.selectorText"
        r = cssutils.css.CSSPageRule()
        r.selectorText = u'a:left'
        self.assertEqual(r.selectorText, u'a:left')
        
        tests = {
            u'': u'',
            u'name': None,
            u':left': None,
            u':right': None,
            u':first': None,
            u':UNKNOWNIDENT': None,
            u'name:left': None,
            u' :left': u':left',
            u':left': u':left',
            u'/*1*/:left/*a*/': u'/*1*/ :left /*a*/',
            u'/*1*/ :left /*a*/ /*b*/': None,
            u':left/*a*/': u':left /*a*/',
            u'/*1*/:left': u'/*1*/ :left',
            }
        self.do_equal_r(tests, att='selectorText')

        tests = {
            u':': xml.dom.SyntaxErr,
            u':/*1*/left': xml.dom.SyntaxErr,
            u': left': xml.dom.SyntaxErr,
            u':left :right': xml.dom.SyntaxErr,
            u':left a': xml.dom.SyntaxErr,
            u'name :left': xml.dom.SyntaxErr,
            }
        self.do_raise_r(tests, att='_setSelectorText')

    def test_specificity(self):
        "CSSPageRule.specificity"
        r = cssutils.css.CSSPageRule()
        tests = {
            u'': (0, 0, 0),
            u'name': (1, 0, 0),
            u':first': (0, 1, 0),
            u':left': (0, 0, 1),
            u':right': (0, 0, 1),
            u':UNKNOWNIDENT': (0, 0, 1),
            u'name:first': (1, 1, 0),
            u'name:left': (1, 0, 1),
            u'name:right': (1, 0, 1),
            u'name:X': (1, 0, 1)
        }
        for sel, exp in tests.items():
            r.selectorText = sel
            self.assertEqual(r.specificity, exp)
            
            r = cssutils.css.CSSPageRule()
            r.cssText = u'@page %s {}' % sel
            self.assertEqual(r.specificity, exp)

    def test_cssRules(self):
        "CSSPageRule.cssRules"
        s = cssutils.parseString(u'@page {}')
        p = s.cssRules[0]
        
        self.assertEqual(len(p.cssRules), 0)
        
        # add and insert
        m1 = cssutils.css.MarginRule(u'@top-left', u'color: red')
        i = p.add(m1)
        self.assertEqual(i, 0)
        self.assertEqual(len(p.cssRules), 1)

        m3 = cssutils.css.MarginRule()
        m3.cssText = u'@top-right { color: blue }'
        i = p.insertRule(m3)
        self.assertEqual(i, 1)
        self.assertEqual(len(p.cssRules), 2)

        m2 = cssutils.css.MarginRule()
        m2.margin = u'@top-center'
        m2.style = u'color: green'
        i = p.insertRule(m2, 1)
        self.assertEqual(i, 1)
        self.assertEqual(len(p.cssRules), 3)

        self.assertEqual(p.cssText, u'''@page {
    @top-left {
        color: red
        }
    @top-center {
        color: green
        }
    @top-right {
        color: blue
        }
    }''')
        
        # keys and dict index
        self.assertEqual(u'@top-left' in p, True)
        self.assertEqual(u'@bottom-left' in p, False)
        
        self.assertEqual(p.keys(), [u'@top-left', 
                                    u'@top-center', 
                                    u'@top-right'])
        
        self.assertEqual(p[u'@bottom-left'], None)
        self.assertEqual(p[u'@top-left'].cssText, u'color: red')
        p[u'@top-left'] = u'color: #f00'
        self.assertEqual(p[u'@top-left'].cssText, u'color: #f00')
        
        # delete
        p.deleteRule(m2)
        self.assertEqual(len(p.cssRules), 2)
        self.assertEqual(p.cssText, u'''@page {
    @top-left {
        color: #f00
        }
    @top-right {
        color: blue
        }
    }''')

        p.deleteRule(0)
        self.assertEqual(len(p.cssRules), 1)        
        self.assertEqual(m3, p.cssRules[0])
        self.assertEqual(p.cssText, u'''@page {
    @top-right {
        color: blue
        }
    }''')

        del p['@top-right']
        self.assertEqual(len(p.cssRules), 0)
                
    
    def test_style(self):
        "CSSPageRule.style (and references)"
        r = cssutils.css.CSSPageRule()
        s1 = r.style
        self.assertEqual(r, s1.parentRule)
        self.assertEqual(u'', s1.cssText)
        
        # set rule.cssText
        r.cssText = '@page { font-family: x1 }'
        self.assertNotEqual(r.style, s1)
        self.assertEqual(r, r.style.parentRule)
        self.assertEqual(r.cssText, u'@page {\n    font-family: x1\n    }')
        self.assertEqual(r.style.cssText, u'font-family: x1')
        self.assertEqual(s1.cssText, u'')
        s2 = r.style
        
        # set invalid rule.cssText
        try: 
            r.cssText = '@page { $ }'
        except xml.dom.SyntaxErr, e:
            pass
        self.assertEqual(r.style, s2)
        self.assertEqual(r, r.style.parentRule)
        self.assertEqual(r.cssText, u'@page {\n    font-family: x1\n    }')
        self.assertEqual(r.style.cssText, u'font-family: x1')
        self.assertEqual(s2.cssText, u'font-family: x1')
        s3 = r.style

        # set rule.style.cssText
        r.style.cssText = 'font-family: x2'
        self.assertEqual(r.style, s3)
        self.assertEqual(r, r.style.parentRule)
        self.assertEqual(r.cssText, u'@page {\n    font-family: x2\n    }')
        self.assertEqual(r.style.cssText, u'font-family: x2')

        # set new style object s2
        s2 = cssutils.css.CSSStyleDeclaration('font-family: y1')
        r.style = s2
        self.assertEqual(r.style, s2)
        self.assertEqual(r, s2.parentRule)
        self.assertEqual(r.cssText, u'@page {\n    font-family: y1\n    }')
        self.assertEqual(s2.cssText, u'font-family: y1')
        self.assertEqual(r.style.cssText, u'font-family: y1')
        self.assertEqual(s3.cssText, u'font-family: x2') # old

        # set s2.cssText
        s2.cssText = 'font-family: y2'
        self.assertEqual(r.style, s2)
        self.assertEqual(r.cssText, u'@page {\n    font-family: y2\n    }')
        self.assertEqual(r.style.cssText, u'font-family: y2')
        self.assertEqual(s3.cssText, u'font-family: x2') # old
        # set invalid s2.cssText
        try: 
            s2.cssText = '$'
        except xml.dom.SyntaxErr, e:
            pass
        self.assertEqual(r.style, s2)
        self.assertEqual(r.cssText, u'@page {\n    font-family: y2\n    }')
        self.assertEqual(r.style.cssText, u'font-family: y2')
        self.assertEqual(s3.cssText, u'font-family: x2') # old

        # set r.style with text
        r.style = 'font-family: z'
        self.assertNotEqual(r.style, s2)
        self.assertEqual(r.cssText, u'@page {\n    font-family: z\n    }')
        self.assertEqual(r.style.cssText, u'font-family: z')

    def test_properties(self):
        "CSSPageRule.style properties"
        r = cssutils.css.CSSPageRule()
        r.style.cssText = '''
        margin-top: 0;
        margin-right: 0;
        margin-bottom: 0;
        margin-left: 0;
        margin: 0;

        page-break-before: auto;
        page-break-after: auto;
        page-break-inside: auto;

        orphans: 3;
        widows: 3;
        '''
        exp = u'''@page {
    margin-top: 0;
    margin-right: 0;
    margin-bottom: 0;
    margin-left: 0;
    margin: 0;
    page-break-before: auto;
    page-break-after: auto;
    page-break-inside: auto;
    orphans: 3;
    widows: 3
    }'''
        self.assertEqual(exp, r.cssText)

    def test_reprANDstr(self):
        "CSSPageRule.__repr__(), .__str__()"
        sel=u':left'
        
        s = cssutils.css.CSSPageRule(selectorText=sel)
        
        self.assertTrue(sel in str(s))

        s2 = eval(repr(s))
        self.assertTrue(isinstance(s2, s.__class__))
        self.assertTrue(sel == s2.selectorText)


if __name__ == '__main__':
    import unittest
    unittest.main()



# -*- coding: utf-8 -*-
"""Testcases for cssutils.helper"""
__version__ = '$Id: test_util.py 1437 2008-08-18 20:30:38Z cthedot $'

import basetest
from cssutils.helper import * 

class HelperTestCase(basetest.BaseTestCase):

    def test_normalize(self):
        "helper._normalize()"
        tests = {u'abcdefg ABCDEFG äöüß€ AÖÜ': ur'abcdefg abcdefg äöüß€ aöü',
                 ur'\ga\Ga\\\ ': ur'gaga\ ',
                 ur'0123456789': ur'0123456789',
                 ur'"\x"': ur'"x"',
                 # unicode escape seqs should have been done by
                 # the tokenizer...
                 }
        for test, exp in tests.items():
            self.assertEqual(normalize(test), exp)
            # static too
            self.assertEqual(normalize(test), exp)

#    def test_normalnumber(self):
#        "helper.normalnumber()"
#        tests = {
#                 '0': '0',
#                 '00': '0',
#                 '0.0': '0',
#                 '00.0': '0',
#                 '1': '1',
#                 '01': '1',
#                 '00.1': '0.1',
#                 '0.00001': '0.00001',
#                 '-0': '0',
#                 '-00': '0',
#                 '-0.0': '0',
#                 '-00.0': '0',
#                 '-1': '-1',
#                 '-01': '-1',
#                 '-00.1': '-0.1',
#                 '-0.00001': '-0.00001',
#                 }
#        for test, exp in tests.items():
#            self.assertEqual(exp, normalnumber(test))

    def test_string(self):
        "helper.string()"
        self.assertEqual(u'"x"', string(u'x'))
        self.assertEqual(u'"1 2ä€"', string(u'1 2ä€'))
        self.assertEqual(ur'''"'"''', string(u"'"))
        self.assertEqual(ur'"\""', string(u'"'))
        # \n = 0xa, \r = 0xd, \f = 0xc
        self.assertEqual(ur'"\a "', string('''
'''))
        self.assertEqual(ur'"\c "', string('\f'))
        self.assertEqual(ur'"\d "', string('\r'))
        self.assertEqual(ur'"\d \a "', string('\r\n'))

    def test_stringvalue(self):
        "helper.stringvalue()"
        self.assertEqual(u'x', stringvalue(u'"x"'))
        self.assertEqual(u'"', stringvalue(u'"\\""'))
        self.assertEqual(ur'x', stringvalue(ur"\x "))
        
        # escapes should have been done by tokenizer
        # so this shoule not happen at all:
        self.assertEqual(ur'a', stringvalue(ur"\a "))

    def test_uri(self):
        "helper.uri()"
        self.assertEqual(u'url(x)', uri('x'))
        self.assertEqual(u'url("(")', uri('('))
        self.assertEqual(u'url(")")', uri(')'))
        self.assertEqual(u'url(" ")', uri(' '))
        self.assertEqual(u'url(";")', uri(';'))
        self.assertEqual(u'url(",")', uri(','))
        self.assertEqual(u'url("x)x")', uri('x)x'))

    def test_urivalue(self):
        "helper.urivalue()"
        self.assertEqual(u'x', urivalue('url(x)'))
        self.assertEqual(u'x', urivalue('url("x")'))
        self.assertEqual(u')', urivalue('url(")")'))


if __name__ == '__main__':
    import unittest
    unittest.main()



"""Testcases for cssutils.codec"""

import codecs
import unittest
import sys

PY2x = sys.version_info < (3,0)
if PY2x:
    import StringIO
    iostream = StringIO.StringIO
else:
    import io
    iostream = io.BytesIO 

from cssutils import codec

try:
    codecs.lookup("utf-32")
except LookupError:
    haveutf32 = False
else:
    haveutf32 = True


class Queue(object):
    """
    queue: write bytes at one end, read bytes from the other end
    """
    def __init__(self):
        self._buffer = "".encode()

    def write(self, chars):
        # TODO ???
        if not PY2x:
            if isinstance(chars, str):
                chars = chars.encode()
            elif isinstance(chars, int):
                chars = bytes([chars])
            
        self._buffer += chars

    def read(self, size=-1):
        if size<0:
            s = self._buffer
            self._buffer = "".encode()
            return s
        else:
            s = self._buffer[:size]
            self._buffer = self._buffer[size:]
            return s


class CodecTestCase(unittest.TestCase):

    def test_detectencoding_str(self):
        "codec.detectencoding_str()"
        self.assertEqual(codec.detectencoding_str(u''.encode()), (None, False))
        self.assertEqual(codec.detectencoding_str(u'\xef'.encode('latin1')), (None, False))
        self.assertEqual(codec.detectencoding_str(u'\xef\x33'.encode("utf-8")), ("utf-8", False))
        self.assertEqual(codec.detectencoding_str(u'\xc3\xaf3'.encode("utf-8")), ("utf-8", False))
        self.assertEqual(codec.detectencoding_str(u'\xef\xbb'.encode("latin1")), (None, False))
        self.assertEqual(codec.detectencoding_str(u'\xef\xbb\x33'.encode("utf-8")), ("utf-8", False))
        self.assertEqual(codec.detectencoding_str(u'\xef\xbb\xbf'.encode("utf-8-sig")), ("utf-8-sig", True))
        self.assertEqual(codec.detectencoding_str(u'\xff'.encode("latin1")), (None, False))
        self.assertEqual(codec.detectencoding_str(u'\xff\x33'.encode("utf-8")), ("utf-8", False))
        self.assertEqual(codec.detectencoding_str(u'\xff\xfe'.encode("latin1")), (None, False))
        self.assertEqual(codec.detectencoding_str(u'\xff\xfe\x33'.encode("utf-16")), ("utf-16", True))
        self.assertEqual(codec.detectencoding_str(u'\xff\xfe\x00'.encode("latin1")), (None, False))
        self.assertEqual(codec.detectencoding_str(u'\xff\xfe\x00\x33'.encode("utf-16")), ("utf-16", True))
        if haveutf32:
            self.assertEqual(codec.detectencoding_str(u'\xff\xfe\x00\x00'.encode("utf-32")), ("utf-32", True))
        self.assertEqual(codec.detectencoding_str(u'\x00'.encode()), (None, False))
        self.assertEqual(codec.detectencoding_str(u'\x00\x33'.encode()), ("utf-8", False))
        self.assertEqual(codec.detectencoding_str(u'\x00\x00'.encode()), (None, False))
        self.assertEqual(codec.detectencoding_str(u'\x00\x00\x33'.encode()), ("utf-8", False))
        self.assertEqual(codec.detectencoding_str(u'\x00\x00\xfe'.encode('latin1')), (None, False))
        self.assertEqual(codec.detectencoding_str(u'\x00\x00\x00\x33'.encode()), ("utf-8", False))
        if haveutf32:
            self.assertEqual(codec.detectencoding_str(u'\x00\x00\x00@'.encode()), ("utf-32-be", False))
            self.assertEqual(codec.detectencoding_str(u'\x00\x00\xfe\xff'.encode('utf-32')), ("utf-32", True))
        self.assertEqual(codec.detectencoding_str(u'@'.encode()), (None, False))
        self.assertEqual(codec.detectencoding_str(u'@\x33'.encode()), ("utf-8", False))
        self.assertEqual(codec.detectencoding_str(u'@\x00'.encode()), (None, False))
        self.assertEqual(codec.detectencoding_str(u'@\x00\x33'.encode()), ("utf-8", False))
        self.assertEqual(codec.detectencoding_str(u'@\x00\x00'.encode()), (None, False))
        self.assertEqual(codec.detectencoding_str(u'@\x00\x00\x33'.encode()), ("utf-8", False))
        if haveutf32:
            self.assertEqual(codec.detectencoding_str(u'@\x00\x00\x00'.encode()), ("utf-32-le", False))
        self.assertEqual(codec.detectencoding_str(u'@c'.encode()), (None, False))
        self.assertEqual(codec.detectencoding_str(u'@ch'.encode()), (None, False))
        self.assertEqual(codec.detectencoding_str(u'@cha'.encode()), (None, False))
        self.assertEqual(codec.detectencoding_str(u'@char'.encode()), (None, False))
        self.assertEqual(codec.detectencoding_str(u'@chars'.encode()), (None, False))
        self.assertEqual(codec.detectencoding_str(u'@charse'.encode()), (None, False))
        self.assertEqual(codec.detectencoding_str(u'@charset'.encode()), (None, False))
        self.assertEqual(codec.detectencoding_str(u'@charset '.encode()), (None, False))
        self.assertEqual(codec.detectencoding_str(u'@charset "'.encode()), (None, False))
        self.assertEqual(codec.detectencoding_str(u'@charset "x'.encode()), (None, False))
        self.assertEqual(codec.detectencoding_str(u'@charset ""'.encode()), ("", True))
        self.assertEqual(codec.detectencoding_str(u'@charset "x"'.encode()), ("x", True))
        self.assertEqual(codec.detectencoding_str(u"@".encode(), False), (None, False))
        self.assertEqual(codec.detectencoding_str(u"@".encode(), True), ("utf-8", False))
        self.assertEqual(codec.detectencoding_str(u"@c".encode(), False), (None, False))
        self.assertEqual(codec.detectencoding_str(u"@c".encode(), True), ("utf-8", False))

    def test_detectencoding_unicode(self):
        "codec.detectencoding_unicode()"
        # Unicode version (only parses the header)
        self.assertEqual(codec.detectencoding_unicode(u'@charset "x'), (None, False))
        self.assertEqual(codec.detectencoding_unicode(u'a {}'), ("utf-8", False))
        self.assertEqual(codec.detectencoding_unicode(u'@charset "x', True), (None, False))
        self.assertEqual(codec.detectencoding_unicode(u'@charset "x"'), ("x", True))

    def test_fixencoding(self):
        "codec._fixencoding()"
        s = u'@charset "'
        self.assertTrue(codec._fixencoding(s, u"utf-8") is None)

        s = u'@charset "x'
        self.assertTrue(codec._fixencoding(s, u"utf-8") is None)

        s = u'@charset "x'
        self.assertEqual(codec._fixencoding(s, u"utf-8", True), s)

        s = u'@charset x'
        self.assertEqual(codec._fixencoding(s, u"utf-8"), s)

        s = u'@charset "x"'
        self.assertEqual(codec._fixencoding(s, u"utf-8"), s.replace('"x"', '"utf-8"'))

    def test_decoder(self):
        "codecs.decoder"
        def checkauto(encoding, input=u'@charset "x";g\xfcrk\u20ac{}'):
            outputencoding = encoding
            if outputencoding == "utf-8-sig":
                outputencoding = "utf-8"
            # Check stateless decoder with encoding autodetection
            d = codecs.getdecoder("css")
            self.assertEqual(d(input.encode(encoding))[0], input.replace('"x"', '"%s"' % outputencoding))

            # Check stateless decoder with specified encoding
            self.assertEqual(d(input.encode(encoding), encoding=encoding)[0], input.replace('"x"', '"%s"' % outputencoding))

            if hasattr(codec, "getincrementaldecoder"):
                # Check incremental decoder with encoding autodetection
                id = codecs.getincrementaldecoder("css")()
                self.assertEqual("".join(id.iterdecode(input.encode(encoding))), input.replace('"x"', '"%s"' % outputencoding))

                # Check incremental decoder with specified encoding
                id = codecs.getincrementaldecoder("css")(encoding=encoding)
                self.assertEqual("".join(id.iterdecode(input.encode(encoding))), input.replace('"x"', '"%s"' % outputencoding))

            # Check stream reader with encoding autodetection
            q = Queue()
            sr = codecs.getreader("css")(q)
            result = []
            # TODO: py3 only???
            for c in input.encode(encoding):
                q.write(c)
                result.append(sr.read())
            self.assertEqual("".join(result), input.replace('"x"', '"%s"' % outputencoding))

            # Check stream reader with specified encoding
            q = Queue()
            sr = codecs.getreader("css")(q, encoding=encoding)
            result = []
            for c in input.encode(encoding):
                q.write(c)
                result.append(sr.read())
            self.assertEqual("".join(result), input.replace('"x"', '"%s"' % outputencoding))

        # Autodetectable encodings
        checkauto("utf-8-sig")
        checkauto("utf-16")
        checkauto("utf-16-le")
        checkauto("utf-16-be")
        if haveutf32:
            checkauto("utf-32")
            checkauto("utf-32-le")
            checkauto("utf-32-be")

        def checkdecl(encoding, input=u'@charset "%s";g\xfcrk{}'):
            # Check stateless decoder with encoding autodetection
            d = codecs.getdecoder("css")
            input = input % encoding
            outputencoding = encoding
            if outputencoding == "utf-8-sig":
                outputencoding = "utf-8"
            self.assertEqual(d(input.encode(encoding))[0], input)

            # Check stateless decoder with specified encoding
            self.assertEqual(d(input.encode(encoding), encoding=encoding)[0], input)

            if hasattr(codec, "getincrementaldecoder"):
                # Check incremental decoder with encoding autodetection
                id = codecs.getincrementaldecoder("css")()
                self.assertEqual("".join(id.iterdecode(input.encode(encoding))), input)

                # Check incremental decoder with specified encoding
                id = codecs.getincrementaldecoder("css")(encoding)
                self.assertEqual("".join(id.iterdecode(input.encode(encoding))), input)

            # Check stream reader with encoding autodetection
            q = Queue()
            sr = codecs.getreader("css")(q)
            result = []
            for c in input.encode(encoding):
                q.write(c)
                result.append(sr.read())
            self.assertEqual("".join(result), input)

            # Check stream reader with specified encoding
            q = Queue()
            sr = codecs.getreader("css")(q, encoding=encoding)
            result = []
            for c in input.encode(encoding):
                q.write(c)
                result.append(sr.read())
            self.assertEqual("".join(result), input)

        # Use correct declaration
        checkdecl("utf-8")
        checkdecl("iso-8859-1", u'@charset "%s";g\xfcrk')
        checkdecl("iso-8859-15")
        checkdecl("cp1252")

        # No recursion
        self.assertRaises(ValueError, u'@charset "css";div{}'.encode().decode, "css")

    def test_encoder(self):
        "codec.encoder"
        def check(encoding, input=u'@charset "x";g\xfcrk\u20ac{}'):
            outputencoding = encoding
            if outputencoding == "utf-8-sig":
                outputencoding = "utf-8"

            # Check stateless encoder with encoding autodetection
            e = codecs.getencoder("css")
            inputdecl = input.replace('"x"', '"%s"' % encoding)
            outputdecl = input.replace('"x"', '"%s"' % outputencoding)
            self.assertEqual(e(inputdecl)[0].decode(encoding), outputdecl)

            # Check stateless encoder with specified encoding
            self.assertEqual(e(input, encoding=encoding)[0].decode(encoding), outputdecl)

            if hasattr(codec, "getincrementalencoder"):
                # Check incremental encoder with encoding autodetection
                ie = codecs.getincrementalencoder("css")()
                self.assertEqual("".join(ie.iterencode(inputdecl)).decode(encoding), outputdecl)

                # Check incremental encoder with specified encoding
                ie = codecs.getincrementalencoder("css")(encoding=encoding)
                self.assertEqual("".join(ie.iterencode(input)).decode(encoding), outputdecl)

            # Check stream writer with encoding autodetection
            q = Queue()
            sw = codecs.getwriter("css")(q)
            for c in inputdecl:#.encode(outputencoding): # TODO: .encode()???
                sw.write(c)
            self.assertEqual(q.read().decode(encoding), input.replace('"x"', '"%s"' % outputencoding))

            # Check stream writer with specified encoding
            q = Queue()
            sw = codecs.getwriter("css")(q, encoding=encoding)
            for c in input:
                sw.write(c)
            self.assertEqual(q.read().decode(encoding), input.replace('"x"', '"%s"' % outputencoding))

        # Autodetectable encodings
        check("utf-8-sig")
        check("utf-16")
        check("utf-16-le")
        check("utf-16-be")
        if haveutf32:
            check("utf-32")
            check("utf-32-le")
            check("utf-32-be")
        check("utf-8")
        check("iso-8859-1", u'@charset "x";g\xfcrk{}')
        check("iso-8859-15")
        check("cp1252")

        # No recursion
        self.assertRaises(ValueError, u'@charset "css";div{}'.encode, "css")

    def test_decode_force(self):
        "codec.decode (force)"
        info = codecs.lookup("css")

        def decodeall(input, **kwargs):
            # Py 2.5: info.decode('@charset "utf-8"; x')
            return info[1](input, **kwargs)[0]

        def incdecode(input, **kwargs):
            decoder = info.incrementaldecoder(**kwargs)
            return decoder.decode(input)

        def streamdecode(input, **kwargs):
            stream = iostream(input) # py3 .decode('utf-8') but still error?!
            reader = info.streamreader(stream, **kwargs)
            return reader.read()

        for d in (decodeall, incdecode, streamdecode):
#            input = '@charset "utf-8"; \xc3\xbf'
#            output = u'@charset "utf-8"; \xff'
#            self.assertEqual(d(input), output)
#
#            input = '@charset "utf-8"; \xc3\xbf'
#            output = u'@charset "iso-8859-1"; \xc3\xbf'
#            self.assertEqual(d(input, encoding="iso-8859-1", force=True), output)
#
#            input = '\xc3\xbf'
#            output = u'\xc3\xbf'
#            self.assertEqual(d(input, encoding="iso-8859-1", force=True), output)
#
#            input = '@charset "utf-8"; \xc3\xbf'
#            output = u'@charset "utf-8"; \xff'
#            self.assertEqual(d(input, encoding="iso-8859-1", force=False), output)

            input = u'@charset "utf-8"; \xff'.encode('utf-8')
            output = u'@charset "utf-8"; \xff'
            self.assertEqual(d(input), output)

            #input = b'@charset "utf-8"; \xc3\xbf'
            input = u'@charset "utf-8"; \xff'.encode('utf-8')
            output = u'@charset "iso-8859-1"; \xc3\xbf'
            self.assertEqual(d(input, encoding="iso-8859-1", force=True), output)

            #input = b'\xc3\xbf'
            input = u'\xff'.encode('utf-8')
            output = u'\xc3\xbf'
            self.assertEqual(d(input, encoding="iso-8859-1", force=True), output)

            #input = b'@charset "utf-8"; \xc3\xbf'
            input = u'@charset "utf-8"; \xff'.encode('utf-8')
            output = u'@charset "utf-8"; \xff'
            self.assertEqual(d(input, encoding="iso-8859-1", force=False), output)


if __name__ == '__main__':
    import unittest
    unittest.main()



"""Implements Document Object Model Level 2 Style Sheets
http://www.w3.org/TR/2000/PR-DOM-Level-2-Style-20000927/stylesheets.html
"""
__all__ = ['MediaList', 'MediaQuery', 'StyleSheet', 'StyleSheetList']
__docformat__ = 'restructuredtext'
__version__ = '$Id$'

from medialist import *
from mediaquery import *
from stylesheet import *
from stylesheetlist import *



"""Testcases for cssutils.css.CSSValue and CSSPrimitiveValue."""
__version__ = '$Id: test_cssvalue.py 1443 2008-08-31 13:54:39Z cthedot $'

import sys
import platform
import basetest
import cssutils

CSS2 = (cssutils.profile.CSS_LEVEL_2,)
C3BUI = (cssutils.profile.CSS3_BASIC_USER_INTERFACE,)
C3BB = (cssutils.profile.CSS3_BACKGROUNDS_AND_BORDERS,)
CM3 = (cssutils.profile.CSS3_COLOR,)
FM3 = (cssutils.profile.CSS3_FONTS,)
C3PM = (cssutils.profile.CSS3_PAGED_MEDIA,)
C3T = (cssutils.profile.CSS3_TEXT,)
FM3FF = (cssutils.profile.CSS3_FONT_FACE,)
CSS2_CM3 = (CM3[0], CSS2[0])
CSS2_FM3 = (FM3[0], CSS2[0])


class ProfilesTestCase(basetest.BaseTestCase):
    M1 = {
          'testvalue': 'x'
          }
    P1 = {
        '-test-tokenmacro': '({num}{w}){1,2}', 
        '-test-macro': '{ident}|{percentage}',
        '-test-custommacro': '{testvalue}',
        # custom validation function 
        '-test-funcval': lambda(v): int(v) > 0 
        }  

    def test_knownNames(self):
        "Profiles.knownNames"
        p = cssutils.profiles.Profiles()
        p.removeProfile(all=True)
        p.addProfile('test', self.P1, self.M1)
        self.assertEqual(p.knownNames, self.P1.keys())
        p.removeProfile(all=True)
        self.assertEqual(p.knownNames, [])
        
    def test_profiles(self):
        "Profiles.profiles"
        p = cssutils.profiles.Profiles()
        p.removeProfile(all=True)
        p.addProfile('test', self.P1, self.M1)
        self.assertEqual(p.profiles, ['test'])
        p.removeProfile(all=True)
        self.assertEqual(p.profiles, [])

    def test_validate2(self):
        "Profiles.validate()"
        #save
        saved = cssutils.profile
        
        # test        
        p = cssutils.profiles.Profiles()
        cssutils.profile = p
        
        pvs = [('color', 'red'),
               ('color', 'rgba(0,0,0,0)'),
               ('color', 'XXX')
               ]
        
        def check(*results):
            for i, pv in enumerate(pvs):
                self.assertEqual(p.validate(*pv), results[i])
                        
        check(True, True, False)

        p.removeProfile(p.CSS3_COLOR)
        check(True, False, False)

        cssutils.profile.addProfile('test', {}, {'color': 'XXX'})
        check(False, False, True)

        p.removeProfile(all=True)
        check(False, False, False)

        # TODO: validateWithProfile
        
        # restore
        cssutils.profile = saved

    def test_addProfile(self):
        "Profiles.addProfile with custom validation function"
        # unknown profile
        self.assertRaises(cssutils.profiles.NoSuchProfileException, 
                          lambda: list(cssutils.profile.propertiesByProfile('NOTSET')) )

        # new profile
        cssutils.profile.addProfile('test', self.P1, self.M1)
        
        props = self.P1.keys()
        props.sort()
        self.assertEqual(props, list(cssutils.profile.propertiesByProfile('test')))
        
        cssutils.log.raiseExceptions = False
        tests = {
            ('-test-tokenmacro', '1'): True,     
            ('-test-tokenmacro', '1 -2'): True,     
            ('-test-tokenmacro', '1 2 3'): False,     
            ('-test-tokenmacro', 'a'): False,     
            ('-test-macro', 'a'): True,     
            ('-test-macro', '0.1%'): True,
            ('-test-custommacro', 'x'): True,     
            ('-test-custommacro', '1'): False,     
            ('-test-custommacro', 'y'): False,     
            ('-test-funcval', '1'): True,     
            ('-test-funcval', '-1'): False,     
            ('-test-funcval', 'x'): False     
            }
        for test, v in tests.items():
            self.assertEqual(v, cssutils.profile.validate(*test))
            
            self.assertEqual((v, v, ['test']), 
                             cssutils.profile.validateWithProfile(*test))
            
        cssutils.log.raiseExceptions = True
        
        # raises:
        expmsg = u"invalid literal for int() with base 10: 'x'"        
        # Python upto 2.4 and Jython have different msg format...
        if sys.version_info[0:2] == (2,4):
            expmsg = u"invalid literal for int(): x" 
        elif sys.platform.startswith('java'):
            expmsg = u"invalid literal for int() with base 10: x"
        # PyPy adds the u prefix, but only in versions lower than Python 3
        elif (platform.python_implementation() == "PyPy" and
              sys.version_info < (3, 0)):
            expmsg = u"invalid literal for int() with base 10: u'x'"

            
        self.assertRaisesMsg(Exception, expmsg, 
                             cssutils.profile.validate, u'-test-funcval', u'x')

    def test_removeProfile(self):
        "Profiles.removeProfile()"
        p = cssutils.profiles.Profiles()
        self.assertEqual(9, len(p.profiles))
        p.removeProfile(p.CSS_LEVEL_2)
        self.assertEqual(8, len(p.profiles))
        p.removeProfile(all=True)
        self.assertEqual(0, len(p.profiles))

    # TODO: FIX
#    def test_validateWithProfile(self):
#        "Profiles.validate(), Profiles.validateWithProfile()"
#        p = cssutils.profiles.Profiles()
#        tests = {
#            ('color', 'red', None): (True, True, [p.CSS_LEVEL_2]),
#            ('color', 'red', p.CSS_LEVEL_2): (True, True,[p.CSS_LEVEL_2]),
#            ('color', 'red', p.CSS3_COLOR): (True, False, [p.CSS_LEVEL_2]),
#            ('color', 'rgba(0,0,0,0)', None): (True, True, [p.CSS3_COLOR]),
#            ('color', 'rgba(0,0,0,0)', p.CSS_LEVEL_2): (True, False, [p.CSS3_COLOR]),
#            ('color', 'rgba(0,0,0,0)', p.CSS3_COLOR): (True, True, [p.CSS3_COLOR]),
#            ('color', '1px', None): (False, False, [p.CSS3_COLOR, p.CSS_LEVEL_2]),
#            ('color', '1px', p.CSS_LEVEL_2): (False, False, [p.CSS3_COLOR, p.CSS_LEVEL_2]),
#            ('color', '1px', p.CSS3_COLOR): (False, False, [p.CSS3_COLOR, p.CSS_LEVEL_2]),
#            ('color', 'aliceblue', None): (True, True, [p.CSS_LEVEL_2]),
#
#            ('opacity', '1', None): (True, True, [p.CSS3_COLOR]),
#            ('opacity', '1', p.CSS_LEVEL_2): (True, False, [p.CSS3_COLOR]),
#            ('opacity', '1', p.CSS3_COLOR): (True, True, [p.CSS3_COLOR]),
#            ('opacity', '1px', None): (False, False, [p.CSS3_COLOR]),
#            ('opacity', '1px', p.CSS_LEVEL_2): (False, False, [p.CSS3_COLOR]),
#            ('opacity', '1px', p.CSS3_COLOR): (False, False, [p.CSS3_COLOR]),
#
#            ('-x', '1', None): (False, False, []),
#            ('-x', '1', p.CSS_LEVEL_2): (False, False, []),
#            ('-x', '1', p.CSS3_COLOR): (False, False, []),
#        }
#        for test, r in tests.items():
#            self.assertEqual(p.validate(test[0], test[1]), r[0])
#            self.assertEqual(p.validateWithProfile(*test), r)
           
    def test_propertiesByProfile(self):
        "Profiles.propertiesByProfile"
        self.assertEqual(['opacity'], #'color',  
                         list(cssutils.profile.propertiesByProfile(
                                            cssutils.profile.CSS3_COLOR)))
        
    def test_csscolorlevel3(self):
        "CSS Color Module Level 3"
        # (propname, propvalue): (valid, validprofile)
        namedcolors = '''transparent, orange,
                         aqua, black, blue, fuchsia, gray, green, lime, maroon,
                         navy, olive, purple, red, silver, teal, white, yellow'''
        for color in namedcolors.split(','):
            color = color.strip()            
            self.assertEqual(True, cssutils.profile.validate('color', color))
            
            self.assertEqual((True, True, list(CSS2)), 
                             cssutils.profile.validateWithProfile('color', color))

            # CSS2 only:
        uicolor = 'ActiveBorder|ActiveCaption|AppWorkspace|Background|ButtonFace|ButtonHighlight|ButtonShadow|ButtonText|CaptionText|GrayText|Highlight|HighlightText|InactiveBorder|InactiveCaption|InactiveCaptionText|InfoBackground|InfoText|Menu|MenuText|Scrollbar|ThreeDDarkShadow|ThreeDFace|ThreeDHighlight|ThreeDLightShadow|ThreeDShadow|Window|WindowFrame|WindowText'
        for color in uicolor.split('|'):
            self.assertEqual(False, cssutils.profile.validate('color', color))
            
            # TODO: Fix
            #self.assertEqual((True, True, list(CSS2)), 
            #                 cssutils.profile.validateWithProfile('color', color))
        
    def test_validate(self):
        "Profiles.validate()"
        tests = {
            # name, values: valid, matching, profile
            
            # background-position
            ('background-position', ('inherit',
                                     '0',
                                     '1%',
                                     '1px',
                                     '0 0',
                                     '1% 1%',
                                     '1px 1px',
                                     '1px 1%',
                                     'top', 'bottom',
                                     'left', 'right',
                                     'center center',
                                     'center',
                                     'top left', 'top center', 'top right',
                                     'bottom left', 'bottom center', 'bottom right',
                                     'center left', 'center center', 'center right',
                                     '0 center', 'center 0',
                                     '0 top', '10% bottom',
                                     'left 0', 'right 10%',
                                     '1% center', 'center 1%'
                                     )): (True, True, CSS2),
            ('background-position', ('0 left',
                                     'top 0'
                                     )): (False, False, CSS2),


            ('border-top-right-radius', ('1px',
                                         '1%',
                                         '1% -1px',
                                         '1% 0',
                                         )): (True, True, C3BB),
            ('border-top-right-radius', ('1px 2px 2px', 
                                         '/ 1px',
                                         'black')): (False, False, C3BB),

            ('border-radius', ('1px',
                               '1%',
                               '0',
                               '1px 1px', 
                               '1px/ 1px', 
                               '1px /1px', 
                               '1px  /  1px', 
                               '1px 1px 1px 1px',
                               '1px 1px 1px 1px / 1px 1px 1px 1px',
                               )): (True, True, C3BB),
            ('border-radius', ('1px /', 
                               '/ 1px',
                               '1px / 1px / 1px',
                               '1px 1px 1px 1px 1px',
                               '1px / 1px 1px 1px 1px 1px',
                               'black')): (False, False, C3BB),

            ('border', ('1px',
                        'solid',
                        'red',
                        '1px solid red',
                        '1px red solid',
                        'red 1px solid',
                        'red solid 1px',
                        'solid 1px red',
                        'solid red 1px',
                               )): (True, True, C3BB),
            ('border', ('1px 1px', 
                        'red red 1px',
                        )): (False, False, C3BB),
            
            ('box-shadow', ('none',
                            '1px 1px', 
                            '1px 1px 1px',
                            '1px 1px 1px 1px',
                            '1px 1px 1px 1px red',
                            'inset 1px 1px',
                            'inset 1px 1px 1px 1px black')): (True, True, C3BB),
            ('box-shadow', ('1px', 
                            '1px 1px 1px 1px 1px',
                            'x 1px 1px',
                            'inset',
                            '1px black',
                            'black')): (False, False, C3BB),
            
            # color
            ('color', ('x',
                       '#',
                       '#0',
                       '#00',
                       '#0000',
                       '#00000',
                       '#0000000',
                       '#00j',
                       '#j00000',
                       'rgb(0.0,1,1)',
                       'rgb(0)',
                       'rgb(0, 1)',
                       'rgb(0, 1, 1, 1)',
                       'rgb(0, 1, 0%)',
                       'rgba(0)',
                       'rgba(0, 1, 1.0, 1)',
                       'rgba(0, 1)',
                       'rgba(0, 1, 1, 1, 1)',
                       'rgba(100%, 0%, 0%, 1%)',
                       'rgba(100%, 0%, 0, 1)',
                       'hsl(1.5,1%,1%)',
                       'hsl(1,1,1%)',
                       'hsl(1,1%,1)',
                       'hsla(1.5,1%,1%, 1)',
                       'hsla(1,1,1%, 1)',
                       'hsla(1,1%,1, 1)',
                       'hsla(1,1%,1%, 1%)'
                       )): (False,False, CSS2_CM3),
            ('color', ('inherit',
                       'black',
                       '#000',
                       '#000000',
                       'rgb(0,1,1)',
                       'rgb( 0 , 1 , 1 )',
                       'rgb(-10,555,1)',
                       'rgb(100%, 1.5%, 0%)',
                       'rgb(150%, -20%, 0%)',
                       )): (True, True, CSS2),
            ('color', ('currentcolor',
                       'aliceblue',
                       'rgba(1,1,1,1)',
                       'rgba( 1 , 1 , 1 , 1 )',
                       'rgba(100%, 0%, 0%, 1)',
                       'hsl(1,1%,1%)',
                       'hsl( 1 , 1% , 1% )',
                       'hsl(-1000,555.5%,-61.5%)',
                       'hsla(1,1%,1%,1)',
                       'hsla( 1, 1% , 1% , 1 )',
                       'hsla(-1000,555.5%,-61.5%, 0.5)'                      
                       )): (True, True, CM3),
            # TODO?:
            #('color', 'rgb(/**/ 0 /**/ , /**/ 1 /**/ , /**/ 1 /**/ )'): (True, True, CSS2),

            # content
            ('content', ('none', 
                         'normal',
                         '""',
                         "'x'",
                         )): (True, True, CSS2),

            ('cursor', ('url(1), auto', 
                        'url(1) 2 3, help',
                        'wait',
                        'inherit',
                        'none')): (True, True, C3BUI),
            ('cursor', ('url(1), auto, wait', 
                        'url(1) 2, help',
                        '1')): (False, False, C3BUI),
            
            # FONTS
            ('font-family', ('serif, x',
                             )): (True, True, CSS2), #CSS2_FM3),
                             
            ('font-family', ('inherit',
                             'a, b',
                             'a,b,c',
                             'a, "b", c',
                             '"a", b, "c"',
                             '"a", "b", "c"',
                             '"x y"',
                             'serif',
                             '"serif"', # valid but CSS2: font with name serif, CSS3: same as `serif`
                             'a  b', # should use quotes but valid
                             'a, b   b, d',
                             )): (True, True, CSS2),

            ('font-weight', ('normal', 'bold', 'bolder', 'lighter', 'inherit', 
                             '100', '200', '300', '400', '500', '600', '700', '800', '900'
                             )): (True, True, CSS2),

            ('font-stretch', ('normal', 'wider', 'narrower', 'ultra-condensed',
                              'extra-condensed', 'condensed', 'semi-condensed',
                              'semi-expanded', 'expanded', 'extra-expanded',
                              'ultra-expanded', 'inherit'                          
                             )): (True, True, FM3),

            ('font-style', ('normal', 'italic', 'oblique', 'inherit'
                             )): (True, True, CSS2),

            ('font-variant', ('normal', 'small-caps', 'inherit'
                             )): (True, True, CSS2),
            ('font-size', ('-1em',
                           )): (False, False, CSS2),
            ('font-size', ('xx-small', 'x-small', 'small', 'medium', 'large', 
                           'x-large', 'xx-large', 'larger', 'smaller', 
                           '1em', '1%', 'inherit'
                           )): (True, True, CSS2),

            ('font-size-adjust', ('1.0', 
                                  'none', 'inherit'
                                  )): (True, True, FM3),

            ('font', ('italic small-caps bold 1px/3 a, "b", serif',
                      '12pt/14pt sans-serif',
                      '80% sans-serif',
                      'x-large/110% "new century schoolbook", serif',
                      'bold italic large Palatino, serif',
                      'normal small-caps 120%/120% fantasy',                     
                      'oblique 12pt "Helvetica Nue", serif', 
                      'caption', 'icon', 'menu', 'message-box', 'small-caption',
                      'status-bar', 'inherit'
                      )): (True, True, CSS2),
            
            ('nav-index', ('1', 'auto', 'inherit')): (True, True, C3BUI),
            ('nav-index', ('x', '1 2', '1px')): (False, False, C3BUI),
            
            ('opacity', ('inherit',
                         '0', '0.0', '0.42342', '1', '1.0',
                         # should be clipped but valid
                         '-0', '-0.1', '-10', '2'
                         )): (True, True, CM3),
            ('opacity', ('a', '#000', '+1')): (False, False, CM3),

            ('outline', ('red dotted 1px', 
                         'dotted 1px red',
                         '1px red dotted',
                         'red', '1px', 'dotted',
                         'red 1px', '1px dotted', 'red dotted',
                         'inherit'
                               )): (True, True, C3BUI),
            ('outline', ('red #fff', 'solid dotted', 'Url(x)', '1px 1px'
                               )): (False, False, C3BUI),
            ('outline-color', ('red', '#fff', 'inherit'
                               )): (True, True, C3BUI),
            ('outline-color', ('0', '1em'
                               )): (False, False, C3BUI),
            ('outline-offset', ('0', '1em', 'inherit'
                               )): (True, True, C3BUI),
            ('outline-offset', ('1%', 'red'
                               )): (False, False, C3BUI),
            ('outline-style', ('auto', 'dotted', 'inherit'
                               )): (True, True, C3BUI),
            ('outline-style', ('0', '1em', 'red'
                               )): (False, False, C3BUI),
            ('outline-width', ('0', '1em', 'inherit'
                               )): (True, True, C3BUI),
            ('outline-width', ('auto', 'red', 'dotted'
                               )): (False, False, C3BUI),
            
            ('resize', ('none', 
                        'both', 
                        'horizontal', 
                        'vertical',
                        'inherit')): (True, True, C3BUI),
            ('resize', ('1', 
                        'auto', 
                        '1px', 
                        '2%')): (False, False, C3BUI),

            ('size', ('1cm',
                      '1mm 20cm',
                      'auto',
                      'landscape letter', 
                      'a4 portrait', 
                      'landscape', 
                      'a5', 
                      #'inherit'
                      )): (True, True, C3PM),
            ('size', ( 
                      'portrait landscape', 
                      'a5 letter', 
                      '2%')): (False, False, C3PM),

            ('src', ('url(  a  )',
                     'local(  x  )',
                     'local("x")',
                     'local(  "x"  )',
                     'url(../fonts/LateefRegAAT.ttf) format(  "truetype-aat"  )',
                     'url(a) format(  "123x"  , "a"   )',
                     'url(a) format( "123x"  , "a"   ), url(a) format( "123x"  , "a"   )',
                     'local(HiraKakuPro-W3), local(Meiryo), local(IPAPGothic)',
                     'local(Gentium), url(/fonts/Gentium.ttf)',
                     'local("Gentium"), url("/fonts/Gentium.ttf")',
                     'local(Futura-Medium), url(fonts.svg#MyGeometricModern) format("svg")',
                    )): (True, True, FM3FF),


            ('text-shadow', ('none',
                            '1px 1px', 
                            '1px 1px 1px',
                            '1px 1px 1px 1px',
                            '1px 1px 1px 1px red',
                            'inset 1px 1px',
                            'inset 1px 1px 1px 1px black')): (True, True, C3T),
            ('text-shadow', ('1px', 
                            '1px 1px 1px 1px 1px',
                            'x 1px 1px',
                            'inset',
                            '1px black',
                            'black')): (False, False, C3T),

            ('unicode-range', ('u+1', 'U+111111-ffffff',
                               'u+123456  ,  U+1-f'
                               )): (True, True, FM3FF),
        
        }
        # TODO!!!
        for (name, values), (valid, matching, profile) in tests.items(): 
            for value in values:
                self.assertEqual(valid, cssutils.profile.validate(name, value))
                
                
#                if (valid, matching, list(profile)) != cssutils.profile.validateWithProfile(name, value):
#                    print
#                    print '###############', name, value
#                    print (valid, matching, list(profile)), cssutils.profile.validateWithProfile(name, value)
             
             # TODO: fix                       
#                self.assertEqual((valid, matching, list(profile)), 
#                                 cssutils.profile.validateWithProfile(name, value))

     # TODO: fix                       
#    def test_validateByProfile(self):
#        "Profiles.validateByProfile()"
#        # testing for valid values overwritten in a profile
#        tests = {
#            (FM3FF, 'font-family', ('y', '"y"' # => name should be "y"!!!
#                                     )): (True, True, FM3FF),    
#            (FM3FF, 'font-family', ('"y", "a"', 'a, b', 'a a'
#                                     )): (True, False, CSS2),    
#            (FM3FF, 'font-stretch', ('normal', 'wider', 'narrower', 'inherit'
#                                     )): (True, False, FM3),    
#            (FM3FF, 'font-style', ('inherit',
#                                     )): (True, False, CSS2),    
#            (FM3FF, 'font-weight', ('bolder', 'lighter', 'inherit',
#                                     )): (True, False, CSS2),    
#            }
#        for (profiles, name, values), (v, m, p) in tests.items():            
#            for value in values:
#                self.assertEqual((v, m, list(p)), 
#                                 cssutils.profile.validateWithProfile(name, 
#                                                                      value, 
#                                                                      profiles))


if __name__ == '__main__':
    import unittest
    unittest.main()
